/*
 * This file is part of the API Extractor project.
 *
 * Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 *
 * Contact: PySide team <contact@pyside.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */

#ifndef TYPEPARSER_H
#define TYPEPARSER_H

#include <QtCore/QList>
#include <QtCore/QString>
#include <QtCore/QStringList>

class TypeParser
{
public:
    struct Info
    {
        Info() : is_reference(false), is_constant(false), is_busted(false), indirections(0) { }
        QStringList qualified_name;
        QStringList arrays;
        QList<Info> template_instantiations;
        uint is_reference : 1;
        uint is_constant : 1;
        uint is_busted : 1;
        uint indirections : 5;

        QString toString() const;
        QString instantiationName() const;
    };

    static Info parse(const QString &str);
};

#endif // TYPEPARSER_H
